@@ -92,7 +92,6 @@ public:
     void queryIcons(QStringList *lst, int size, KIconLoader::Context context) const;
     void queryIconsByContext(QStringList *lst, int size, KIconLoader::Context context) const;
     QString findIcon(const QString &name, int size, KIconLoader::MatchType match) const;
-    void printTree(QString &dbgString) const;
 
     KIconTheme *theme;
 };
@@ -107,15 +106,6 @@ KIconThemeNode::~KIconThemeNode()
     delete theme;
 }
 
-void KIconThemeNode::printTree(QString &dbgString) const
-{
-    /* This method doesn't have much sense anymore, so maybe it should
-       be removed in the (near?) future */
-    dbgString += '(';
-    dbgString += theme->name();
-    dbgString += ')';
-}
-
 void KIconThemeNode::queryIcons(QStringList *result,
                                 int size, KIconLoader::Context context) const
 {
@@ -171,6 +161,8 @@ public:
         appname.clear();
         searchPaths.clear();
         links.clear();
+        mIconThemeInited = false;
+        mThemesInTree.clear();
     }
 
     /**
@@ -325,19 +317,19 @@ class KIconLoaderGlobalData : public QObject
 public:
     KIconLoaderGlobalData()
     {
-        const QStringList genericIconsFiles = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "mime/generic-icons");
+        const QStringList genericIconsFiles = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("mime/generic-icons"));
         //qCDebug(KICONTHEMES) << genericIconsFiles;
         Q_FOREACH (const QString &file, genericIconsFiles) {
             parseGenericIconsFiles(file);
         }
 
-        QDBusConnection::sessionBus().connect(QString(), "/KIconLoader", "org.kde.KIconLoader",
-                                              "iconChanged", this, SIGNAL(iconChanged(int)));
+        QDBusConnection::sessionBus().connect(QString(), QStringLiteral("/KIconLoader"), QStringLiteral("org.kde.KIconLoader"),
+                                              QStringLiteral("iconChanged"), this, SIGNAL(iconChanged(int)));
     }
 
     void emitChange(KIconLoader::Group group)
     {
-        QDBusMessage message = QDBusMessage::createSignal("/KIconLoader", "org.kde.KIconLoader", "iconChanged");
+        QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/KIconLoader"), QStringLiteral("org.kde.KIconLoader"), QStringLiteral("iconChanged"));
         message.setArguments(QList<QVariant>() << int(group));
         QDBusConnection::sessionBus().send(message);
     }
@@ -498,7 +490,7 @@ void KIconLoaderPrivate::init(const QString &_appname, const QStringList &extraS
     }
 
     // Initialize icon cache
-    mIconCache = new KSharedDataCache("icon-cache", 10 * 1024 * 1024);
+    mIconCache = new KSharedDataCache(QStringLiteral("icon-cache"), 10 * 1024 * 1024);
     // Cost here is number of pixels, not size. So this is actually a bit
     // smaller.
     mPixmapCache.setMaxCost(10 * 1024 * 1024);
@@ -556,15 +548,9 @@ bool KIconLoaderPrivate::initIconThemes()
     searchPaths.append(appname + "/pics");
 
     // Add legacy icon dirs.
-    searchPaths.append("icons"); // was xdgdata-icon in KStandardDirs
+    searchPaths.append(QStringLiteral("icons")); // was xdgdata-icon in KStandardDirs
     // These are not in the icon spec, but e.g. GNOME puts some icons there anyway.
-    searchPaths.append("pixmaps"); // was xdgdata-pixmaps in KStandardDirs
-
-#ifndef NDEBUG
-    QString dbgString = "Theme tree: ";
-    mpThemeRoot->printTree(dbgString);
-    qCDebug(KICONTHEMES) << dbgString;
-#endif
+    searchPaths.append(QStringLiteral("pixmaps")); // was xdgdata-pixmaps in KStandardDirs
 
     return true;
 }
@@ -591,7 +577,7 @@ void KIconLoaderPrivate::addAppThemes(const QString &appname, const QString &the
 {
     initIconThemes();
 
-    KIconTheme *def = new KIconTheme("hicolor", appname, themeBaseDir);
+    KIconTheme *def = new KIconTheme(QStringLiteral("hicolor"), appname, themeBaseDir);
     if (!def->isValid()) {
         delete def;
         def = new KIconTheme(KIconTheme::defaultThemeName(), appname, themeBaseDir);
@@ -626,7 +612,7 @@ void KIconLoaderPrivate::addBaseThemes(KIconThemeNode *node, const QString &appn
 
     addInheritedThemes(node, appname);
     addThemeByName(KIconTheme::defaultThemeName(), appname);
-    addThemeByName("hicolor", appname);
+    addThemeByName(QStringLiteral("hicolor"), appname);
 }
 
 void KIconLoaderPrivate::addInheritedThemes(KIconThemeNode *node, const QString &appname)
@@ -634,7 +620,7 @@ void KIconLoaderPrivate::addInheritedThemes(KIconThemeNode *node, const QString
     const QStringList lst = node->theme->inherits();
 
     for (QStringList::ConstIterator it = lst.begin(); it != lst.end(); ++it) {
-        if ((*it) == "hicolor") {
+        if ((*it) == QLatin1String("hicolor")) {
             // The icon theme spec says that "hicolor" must be the very last
             // of all inherited themes, so don't add it here but at the very end
             // of addBaseThemes().
@@ -669,7 +655,7 @@ void KIconLoaderPrivate::addExtraDesktopThemes()
     initIconThemes();
 
     QStringList list;
-    const QStringList icnlibs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "icons", QStandardPaths::LocateDirectory);
+    const QStringList icnlibs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("icons"), QStandardPaths::LocateDirectory);
     QStringList::ConstIterator it;
     char buf[1000];
     for (it = icnlibs.begin(); it != icnlibs.end(); ++it) {
@@ -677,7 +663,7 @@ void KIconLoaderPrivate::addExtraDesktopThemes()
         if (!dir.exists()) {
             continue;
         }
-        const QStringList lst = dir.entryList(QStringList("default.*"), QDir::Dirs);
+        const QStringList lst = dir.entryList(QStringList(QStringLiteral("default.*")), QDir::Dirs);
         QStringList::ConstIterator it2;
         for (it2 = lst.begin(); it2 != lst.end(); ++it2) {
             if (!QFile::exists(*it + *it2 + "/index.desktop")
@@ -706,7 +692,7 @@ void KIconLoaderPrivate::addExtraDesktopThemes()
                 || *it == QLatin1String("default.kde4")) {
             continue;
         }
-        addThemeByName(*it, "");
+        addThemeByName(*it, QLatin1String(""));
     }
 
     extraDesktopIconsLoaded = true;
@@ -776,7 +762,7 @@ QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group
            % QLatin1Char('_')
            % QString::number(size)
            % QLatin1Char('_')
-           % overlays.join("_")
+           % overlays.join(QStringLiteral("_"))
            % (group >= 0 ? mpEffect.fingerprint(group, state)
               : *NULL_EFFECT_FINGERPRINT());
 }
@@ -900,58 +886,24 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) cons
 {
     const_cast<KIconLoaderPrivate *>(this)->initIconThemes();
 
-    QString path;
-
-    const char *const ext[4] = { ".png", ".svgz", ".svg", ".xpm" };
-    bool genericFallback = name.endsWith(QLatin1String("-x-generic"));
-
-    // Do two passes through themeNodes.
-    //
-    // The first pass looks for an exact match in each themeNode one after the other.
-    // If one is found and it is an app icon then return that icon.
-    //
-    // In the next pass (assuming the first pass failed), it looks for exact matches
-    // and then generic fallbacks in each themeNode one after the other
-    //
-    // The reasoning is that application icons should always match exactly, all other
-    // icons may fallback. Since we do not know what the context is here when we start
-    // looking for it, we can only go by the path found.
     foreach (KIconThemeNode *themeNode, links) {
-        for (int i = 0; i < 4; i++) {
-            path = themeNode->theme->iconPath(name + ext[i], size, KIconLoader::MatchExact);
-            if (!path.isEmpty()) {
-                return path;
-            }
-
-            path = themeNode->theme->iconPath(name + ext[i], size, KIconLoader::MatchBest);
-            if (!path.isEmpty()) {
-                return path;
-            }
-        }
-
-        if (!path.isEmpty() && path.contains("/apps/")) {
+        const QString path = themeNode->theme->iconPathByName(name, size, KIconLoader::MatchBest);
+        if (!path.isEmpty()) {
             return path;
         }
     }
 
+    bool genericFallback = name.endsWith(QLatin1String("-x-generic"));
+    QString path;
     foreach (KIconThemeNode *themeNode, links) {
         QString currentName = name;
 
         while (!currentName.isEmpty()) {
             //qCDebug(KICONTHEMES) << "Looking up" << currentName;
 
-            for (int i = 0; i < 4; i++) {
-                path = themeNode->theme->iconPath(currentName + ext[i], size, KIconLoader::MatchExact);
-                if (!path.isEmpty()) {
-                    return path;
-                }
-
-                path = themeNode->theme->iconPath(currentName + ext[i], size, KIconLoader::MatchBest);
-                if (!path.isEmpty()) {
-                    return path;
-                }
-            }
-            //qCDebug(KICONTHEMES) << "Looking up" << currentName;
+            path = themeNode->theme->iconPathByName(currentName, size, KIconLoader::MatchBest);
+            if (!path.isEmpty())
+                return path;
 
             if (genericFallback) {
                 // we already tested the base name
@@ -968,9 +920,9 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) cons
             } else {
                 // From update-mime-database.c
                 static const QSet<QString> mediaTypes = QSet<QString>()
-                                                        << "text" << "application" << "image" << "audio"
-                                                        << "inode" << "video" << "message" << "model" << "multipart"
-                                                        << "x-content" << "x-epoc";
+                                                        << QStringLiteral("text") << QStringLiteral("application") << QStringLiteral("image") << QStringLiteral("audio")
+                                                        << QStringLiteral("inode") << QStringLiteral("video") << QStringLiteral("message") << QStringLiteral("model") << QStringLiteral("multipart")
+                                                        << QStringLiteral("x-content") << QStringLiteral("x-epoc");
                 // Shared-mime-info spec says:
                 // "If [generic-icon] is not specified then the mimetype is used to generate the
                 // generic icon by using the top-level media type (e.g. "video" in "video/ogg")
@@ -989,7 +941,7 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) cons
 
 inline QString KIconLoaderPrivate::unknownIconPath(int size) const
 {
-    QString path = findMatchingIcon(QLatin1String("unknown"), size);
+    QString path = findMatchingIcon(QStringLiteral("unknown"), size);
     if (path.isEmpty()) {
         qCDebug(KICONTHEMES) << "Warning: could not find \"unknown\" icon for size" << size;
         return QString();
@@ -1099,7 +1051,7 @@ QPixmap KIconLoader::loadMimeTypeIcon(const QString &_iconName, KIconLoader::Gro
     const QPixmap pixmap = loadIcon(iconName, group, size, state, overlays, path_store, true);
     if (pixmap.isNull()) {
         // Icon not found, fallback to application/octet-stream
-        return loadIcon("application-octet-stream", group, size, state, overlays, path_store, false);
+        return loadIcon(QStringLiteral("application-octet-stream"), group, size, state, overlays, path_store, false);
     }
     return pixmap;
 }
@@ -1174,7 +1126,7 @@ QPixmap KIconLoader::loadIcon(const QString &_name, KIconLoader::Group group, in
         if (absolutePath && !favIconOverlay) {
             path = name;
         } else {
-            path = d->findMatchingIconWithGenericFallbacks(favIconOverlay ? QString("text-html") : name, size);
+            path = d->findMatchingIconWithGenericFallbacks(favIconOverlay ? QStringLiteral("text-html") : name, size);
         }
     }
 
@@ -1348,7 +1300,7 @@ QStringList KIconLoader::loadAnimated(const QString &name, KIconLoader::Group gr
     }
 
     foreach (const QString &entry, dir.entryList()) {
-        if (!(entry.left(4)).toUInt()) {
+        if (!(entry.leftRef(4)).toUInt()) {
             continue;
         }
 
@@ -1383,7 +1335,7 @@ int KIconLoader::currentSize(KIconLoader::Group group) const
 QStringList KIconLoader::queryIconsByDir(const QString &iconsDir) const
 {
     const QDir dir(iconsDir);
-    const QStringList formats = QStringList() << "*.png" << "*.xpm" << "*.svg" << "*.svgz";
+    const QStringList formats = QStringList() << QStringLiteral("*.png") << QStringLiteral("*.xpm") << QStringLiteral("*.svg") << QStringLiteral("*.svgz");
     const QStringList lst = dir.entryList(formats, QDir::Files);
     QStringList result;
     QStringList::ConstIterator it;
@@ -1608,17 +1560,17 @@ int IconSize(KIconLoader::Group group)
 QPixmap KIconLoader::unknown()
 {
     QPixmap pix;
-    if (QPixmapCache::find("unknown", pix)) { //krazy:exclude=iconnames
+    if (QPixmapCache::find(QStringLiteral("unknown"), pix)) { //krazy:exclude=iconnames
         return pix;
     }
 
-    QString path = global()->iconPath("unknown", KIconLoader::Small, true); //krazy:exclude=iconnames
+    QString path = global()->iconPath(QStringLiteral("unknown"), KIconLoader::Small, true); //krazy:exclude=iconnames
     if (path.isEmpty()) {
         qCDebug(KICONTHEMES) << "Warning: Cannot find \"unknown\" icon.";
         pix = QPixmap(32, 32);
     } else {
         pix.load(path);
-        QPixmapCache::insert("unknown", pix); //krazy:exclude=iconnames
+        QPixmapCache::insert(QStringLiteral("unknown"), pix); //krazy:exclude=iconnames
     }
 
     return pix;
